﻿using System;

namespace BadStaticFieldInit
{
    // 例3-44 static フィールド初期化子の順序に対する配慮
    class Bar
    {
        public bool myField;
    }

    // 本文中には"#if" "#else" "#endif" がありませんが、
    // 以下のコードでは例3-44の2つのケースを試すことができるよう
    // 追加しています。
    // "false" を "true" に変更すると1番目のコードが有効になり、
    // 実行するとアプリケーションがクラッシュすることが確認できます。
#if false
    // 悪い例 - 初期化中にnull参照例外が発生します。
    class Foo
    {
      public static bool field2 = field1.myField;
      public static Bar field1 = new Bar();
    }
#else
    // 良い例 - 正しい順序で初期化されています。
    class Foo
    {
        public static Bar field1 = new Bar();
        public static bool field2 = field1.myField;
    }
#endif

    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine(Foo.field2);

            // C#のコンパイラが警告を出さないようにするためのコードです。
            // C#は全く変更されないフィールドが定義された場合にそれを検出
            // するようになっています(通常はこのように警告を取り消すだけが(
            // 目的となるようなコードを作成することはないはずです。
            // というのも、警告が出るということは何かしらの問題が
            // 発生していることを表しているからです。今回の場合は
            // C#の特定の機能に注目してもらい、その他の要素を排除するために
            // あえてこのようなコードにしています)。
            Foo.field1.myField = true;
        }
    }
}
