﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentProcessing
{
    class Program
    {
        static void Main(string[] args)
        {
            Document doc1 = new Document
            {
                Author = "Matthew Adams",
                DocumentDate = new DateTime(2000, 01, 01),
                Text = "Am I a year early?"
            };
            Document doc2 = new Document
            {
                Author = "Ian Griffiths",
                DocumentDate = new DateTime(2001, 01, 01),
                Text = "This is the new millenium, I promise you."
            };
            Document doc3 = new Document
            {
                Author = "Matthew Adams",
                DocumentDate = new DateTime(2002, 01, 01),
                Text = "Another year, another document."
            };

           
            DocumentProcessor processor = Configure();

            string documentBeingProcessed = "(文書未設定)";

            int processCount = 0;
            processor.LogTextProvider = (doc =>
            {
                processCount += 1;
                return documentBeingProcessed;
            });

            ProductionDeptTool1 tool1 = new ProductionDeptTool1();
            tool1.Subscribe(processor);

            ProductionDeptTool2 tool2 = new ProductionDeptTool2();
            tool2.Subscribe(processor);

            documentBeingProcessed = "(文書1)";
            processor.Process(doc1);
            Console.WriteLine();

            documentBeingProcessed = "(文書2)";
            processor.Process(doc2);
            Console.WriteLine();

            documentBeingProcessed = "(文書3)";
            processor.Process(doc3);

            Console.WriteLine();
            Console.WriteLine(processCount + "個の処理を実行しました。");

            Console.ReadKey();
        }

        static DocumentProcessor Configure()
        {
            DocumentProcessor rc = new DocumentProcessor();
            rc.AddProcess(
                DocumentProcesses.TranslateIntoFrench,
                doc => !doc.Text.Contains("?"));
            rc.AddProcess(DocumentProcesses.Spellcheck);
            rc.AddProcess(DocumentProcesses.Repaginate);

            TrademarkFilter trademarkFilter = new TrademarkFilter();
            trademarkFilter.Trademarks.Add("Ian");
            trademarkFilter.Trademarks.Add("Griffiths");
            trademarkFilter.Trademarks.Add("millenium");

            rc.AddProcess(trademarkFilter.HighlightTrademarks);

            return rc;
        }
    }
}
