﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentProcessing
{
    class DocumentProcessor
    {
        // 例5-25 イベントメンバーの更新
        public event EventHandler<ProcessEventArgs> Processing;
        public event EventHandler<ProcessEventArgs> Processed;

        public Func<Document, string> LogTextProvider
        {
            get;
            set;
        }


        class ActionCheckPair
        {
            public Action<Document> Action { get; set; }
            public Predicate<Document> QuickCheck { get; set; }
        }

        private readonly List<ActionCheckPair> processes =
            new List<ActionCheckPair>();

        public void AddProcess(Action<Document> action)
        {
            AddProcess(action, null);
        }

        public void AddProcess(Action<Document> action,
            Predicate<Document> quickCheck)
        {
            processes.Add(
            new ActionCheckPair { Action = action, QuickCheck = quickCheck });
        }

        // 例5-27 イベント引数オブジェクトの生成
        public void Process(Document doc)
        {
            ProcessEventArgs e = new ProcessEventArgs(doc);
            OnProcessing(e);
            
            // 最初に簡易チェックを行います。
            foreach (ActionCheckPair process in processes)
            {
                if (process.QuickCheck != null && !process.QuickCheck(doc))
                {
                    Console.WriteLine("処理は成功しないでしょう。");
                    if (LogTextProvider != null)
                    {
                        Console.WriteLine(LogTextProvider(doc));
                    }
                    OnProcessed(e);
                    return;
                }
            }

            // 次にアクションを実行します。
            foreach (ActionCheckPair process in processes)
            {
                process.Action(doc);
                if (LogTextProvider != null)
                {
                    Console.WriteLine(LogTextProvider(doc));
                }
            }
            OnProcessed(e);
        }

        // 例5-26 イベントを発生させるコードの更新
        private void OnProcessing(ProcessEventArgs e)
        {
            if (Processing != null)
            {
                Processing(this, e);
            }
        }

        private void OnProcessed(ProcessEventArgs e)
        {
            if (Processed != null)
            {
                Processed(this, e);
            }
        }
    }
}