﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentProcessing
{
    class DocumentProcessor
    {
        public event EventHandler Processing;
        public event EventHandler Processed;

        public Func<Document, string> LogTextProvider
        {
            get;
            set;
        }


        class ActionCheckPair
        {
            public Action<Document> Action { get; set; }
            public Predicate<Document> QuickCheck { get; set; }
        }

        private readonly List<ActionCheckPair> processes =
            new List<ActionCheckPair>();

        public void AddProcess(Action<Document> action)
        {
            AddProcess(action, null);
        }

        public void AddProcess(Action<Document> action,
            Predicate<Document> quickCheck)
        {
            processes.Add(
            new ActionCheckPair { Action = action, QuickCheck = quickCheck });
        }

        // 例5-21 イベントの発生
        public void Process(Document doc)
        {
            OnProcessing(EventArgs.Empty);
            // 最初に簡易チェックを行います。
            foreach (ActionCheckPair process in processes)
            {
                if (process.QuickCheck != null && !process.QuickCheck(doc))
                {
                    Console.WriteLine("処理は成功しないでしょう。");
                    if (LogTextProvider != null)
                    {
                        Console.WriteLine(LogTextProvider(doc));
                    }
                    OnProcessed(EventArgs.Empty);
                    return;
                }
            }

            // 次にアクションを実行します。
            foreach (ActionCheckPair process in processes)
            {
                process.Action(doc);
                if (LogTextProvider != null)
                {
                    Console.WriteLine(LogTextProvider(doc));
                }
            }
            OnProcessed(EventArgs.Empty);
        }

        private void OnProcessing(EventArgs e)
        {
            if (Processing != null)
            {
                Processing(this, e);
            }
        }

        private void OnProcessed(EventArgs e)
        {
            if (Processed != null)
            {
                Processed(this, e);
            }
        }
    }
}