﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentProcessing
{
    // 例5-13 簡易チェック機能を文書プロセッサーに追加
    class DocumentProcessor
    {
        class ActionCheckPair
        {
            public Action<Document> Action { get; set; }
            public Check QuickCheck { get; set; }
        }

        private readonly List<ActionCheckPair> processes = new List<ActionCheckPair>();


        public void AddProcess(Action<Document> action)
        {
            AddProcess(action, null);
        }

        public void AddProcess(Action<Document> action, Check quickCheck)
        {
            processes.Add(
            new ActionCheckPair { Action = action, QuickCheck = quickCheck });
        }

        public void Process(Document doc)
        {
            // 最初に簡易チェックを行います。
            foreach (ActionCheckPair process in processes)
            {
                if (process.QuickCheck != null && !process.QuickCheck(doc))
                {
                    Console.WriteLine("処理は成功しないでしょう。");
                    return;
                }
            }

            // 次にアクションを実行します。
            foreach (ActionCheckPair process in processes)
            {
                process.Action(doc);
            }
        }
    }
}