﻿using System;

namespace ArraySize
{
    class Program
    {
        static void Main(string[] args)
        {
            CalendarEvent[] events =
            {
                new CalendarEvent
                {
                    Title = "サウスバンクでスウィング・ダンス",
                    StartTime = new DateTimeOffset (2009, 7, 11, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "サタデー・ナイト・スウィング",
                    StartTime = new DateTimeOffset (2009, 7, 11, 19, 30, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(6.5)
                },
                new CalendarEvent
                {
                    Title = "F1 ドイツグランプリ",
                    StartTime = new DateTimeOffset (2009, 7, 12, 12, 10, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(3)
                },
                new CalendarEvent
                {
                    Title = "スウィング・ダンス・ピクニック",
                    StartTime = new DateTimeOffset (2009, 7, 12, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "100 Clubでのスウィングダンスイベント",
                    StartTime = new DateTimeOffset (2009, 7, 13, 19, 45, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(5)
                }
            };

            foreach (CalendarEvent[] day in GetEventsByDay(events, new DateTime(2009, 7, 11), 5))
            {
                Console.WriteLine("-----");
                PrintEvents(day);
            }

            // 例7-21 多次元の四角形配列
            int[,] walls = new int[,]
            {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1 },
                { 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
            };


            Console.WriteLine(CanCharacterMoveDown(0, 1, walls));
            Console.WriteLine(CanCharacterMoveDown(1, 1, walls));
            Console.WriteLine(CanCharacterMoveDown(3, 1, walls));
            Console.WriteLine(CanCharacterMoveDown(9, 1, walls));
            Console.WriteLine(CanCharacterMoveDown(10, 1, walls));
            Console.WriteLine(CanCharacterMoveDown(10, 4, walls));
            Console.WriteLine(CanCharacterMoveDown(10, 5, walls));
        }

        // 例7-20 配列の配列の生成
        static CalendarEvent[][] GetEventsByDay(CalendarEvent[] allEvents,
                                                DateTime firstDay,
                                                int numberOfDays)
        {
            CalendarEvent[][] eventsByDay = new CalendarEvent[numberOfDays][];

            for (int day = 0; day < numberOfDays; ++day)
            {
                DateTime dateOfInterest = (firstDay + TimeSpan.FromDays(day)).Date;
                CalendarEvent[] itemsOnDateOfInterest = Array.FindAll(allEvents,
                e => e.StartTime.Date == dateOfInterest);
                eventsByDay[day] = itemsOnDateOfInterest;
            }

            return eventsByDay;
        }

        static void PrintEvents(CalendarEvent[] events)
        {
            foreach (CalendarEvent item in events)
            {
                Console.WriteLine(item.Title + ": " + item.StartTime);
            }
        }

        // 例7-22 四角形配列の要素の取得
        static bool CanCharacterMoveDown(int x, int y, int[,] walls)
        {
            int newY = y + 1;

            // マップの一番下からさらに下に動くことはできない
            if (newY == walls.GetLength(0)) { return false; }

            // 通り道に壁がないときだけ下に動くことができる
            return walls[newY, x] == 0;
        }
    }
}
