﻿using System;
using System.Collections.Generic;
using System.Diagnostics;

class Program
{
    static void Main(string[] args)
    {
        // 例7-25 独自のインデクサの使用
        Indexable ix = new Indexable();
        Console.WriteLine(ix[10]);
        ix[42] = "Xyzzy";

        ImmutableValue();
        MutableValue();

        MicroBenchmarkList();
    }

    static void ImmutableValue()
    {
        ArrayAndIndexer<int> aai = new ArrayAndIndexer<int>();
        aai.TheArray[10] = 42;
        Console.WriteLine(aai[10]);
        aai[20] = 99;
        Console.WriteLine(aai.TheArray[20]);
    }

    static void MutableValue()
    {
        ArrayAndIndexer<CanChange> aai = new ArrayAndIndexer<CanChange>();
        aai.TheArray[10] = new CanChange { Number = 42 };
        Console.WriteLine(aai[10].Number);
        aai[20] = new CanChange { Number = 99, Name = "独自の要素" };
        Console.WriteLine(aai.TheArray[20].Number);

        aai.TheArray[10].Number = 123;
        Console.WriteLine(aai.TheArray[10].Number);

        // コンパイルできません。コメントアウトを解除すると確認できます。
        //aai[20].Number = 456;

        // 例7-27 コンパイラの解釈と同等なコード
        CanChange elem = aai[20];
        elem.Number = 456;
    }


    static void MicroBenchmarkList()
    {
        // 例7-28 値型リスト対参照型リストの簡単なベンチマーク
        // (パフォーマンスを比較するために、CanChangeの型をclassまたはstructに切り替えて実行してみてください)
        Stopwatch sw = new Stopwatch();
        sw.Start();
        int itemCount = 500000;
        List<CanChange> items = new List<CanChange>(itemCount);
        for (int i = 0; i < itemCount; ++i)
        {
            items.Add(new CanChange { Number = i });
        }
        sw.Stop();
        Console.WriteLine("作成時間: " + sw.ElapsedTicks);
        sw.Reset();
        sw.Start();
        int total = 0;
        for (int i = 0; i < itemCount; ++i)
        {
            total += items[i].Number;
        }
        sw.Stop();
        Console.WriteLine("総和: " + total);
        Console.WriteLine("計算時間: " + sw.ElapsedTicks);
    }
}
