﻿using System;
using System.Collections.Generic;
using System.IO;

namespace LazyEnumeration
{
    // 例7-34 巨大で時間のかかるデータセットに対する遅延的反復処理
    class Program
    {
        static IEnumerable<string> GetAllFilesInDirectory(string directoryPath)
        {
            IEnumerable<string> files = null;
            IEnumerable<string> subdirectories = null;
            try
            {
                files = Directory.EnumerateFiles(directoryPath);
                subdirectories = Directory.EnumerateDirectories(directoryPath);
            }
            catch (UnauthorizedAccessException)
            {
                Console.WriteLine("アクセス権限がありません: " + directoryPath);
            }
            if (files != null)
            {
                foreach (string file in files)
                {
                    yield return file;
                }
            }
            if (subdirectories != null)
            {
                foreach (string subdirectory in subdirectories)
                {
                    foreach (string file in GetAllFilesInDirectory(subdirectory))
                    {
                        yield return file;
                    }
                }
            }
        }
        static void Main(string[] args)
        {
            // 例を切り替えて実行するにはtrueをfalseに変更します。
#if true
            foreach (string file in GetAllFilesInDirectory(@"c:\"))
            {
                Console.WriteLine(file);
            }
#else
            // 例7-35 遅延的反復処理の連結
            IEnumerable<string> allFiles = GetAllFilesInDirectory(@"c:\");
            IEnumerable<string> numberedFiles = AddNumbers(allFiles);
            foreach (string file in numberedFiles)
            {
                Console.WriteLine(file);
            }
#endif
        }

        static IEnumerable<string> AddNumbers(IEnumerable<string> names)
        {
            int i = 0;
            foreach (string currentName in names)
            {
                yield return string.Format("{0}: {1}", i, currentName);
                i += 1;
            }
        }

    }
}
