﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Grouping
{
    class Program
    {
        static void Main(string[] args)
        {
            List<CalendarEvent> events = new List<CalendarEvent>
            {
                new CalendarEvent
                {
                    Title = "サウスバンクでスウィング・ダンス",
                    StartTime = new DateTimeOffset (2009, 7, 11, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "サタデー・ナイト・スウィング",
                    StartTime = new DateTimeOffset (2009, 7, 11, 19, 30, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(6.5)
                },
                new CalendarEvent
                {
                    Title = "F1 ドイツグランプリ",
                    StartTime = new DateTimeOffset (2009, 7, 12, 12, 10, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(3)
                },
                new CalendarEvent
                {
                    Title = "スウィング・ダンス・ピクニック",
                    StartTime = new DateTimeOffset (2009, 7, 12, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "100 Clubでのスウィングダンスイベント",
                    StartTime = new DateTimeOffset (2009, 7, 13, 19, 45, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(5)
                }
            };


            // 例8-15 LINQによる単純なグループ化
            var eventsByDay = from ev in events
                              group ev by ev.StartTime.Date;

            // 例8-16 グループ化された結果を走査
            foreach (var day in eventsByDay)
            {
                Console.WriteLine(day.Key + " のイベント");
                foreach (var item in day)
                {
                    Console.WriteLine(item.Title);
                }
            }

            UsingInto(events);
        }

        static void UsingInto(List<CalendarEvent> events)
        {
            // 例8-17 intoを指定してグループ化を行う
            var eventsByDay = from ev in events
                              group ev by ev.StartTime.Date into dayGroup
                              select dayGroup.ToArray();

            foreach (var day in eventsByDay)
            {
                Console.WriteLine();
                foreach (var item in day)
                {
                    Console.WriteLine(item.Title);
                }
            }
        }
    }
}
