﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Projections
{
    class Program
    {
        static void Main(string[] args)
        {
            List<CalendarEvent> events = new List<CalendarEvent>
            {
                new CalendarEvent
                {
                    Title = "サウスバンクでスウィング・ダンス",
                    StartTime = new DateTimeOffset (2009, 7, 11, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "サタデー・ナイト・スウィング",
                    StartTime = new DateTimeOffset (2009, 7, 11, 19, 30, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(6.5)
                },
                new CalendarEvent
                {
                    Title = "F1 ドイツグランプリ",
                    StartTime = new DateTimeOffset (2009, 7, 12, 12, 10, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(3)
                },
                new CalendarEvent
                {
                    Title = "スウィング・ダンス・ピクニック",
                    StartTime = new DateTimeOffset (2009, 7, 12, 15, 00, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(4)
                },
                new CalendarEvent
                {
                    Title = "100 Clubでのスウィングダンスイベント",
                    StartTime = new DateTimeOffset (2009, 7, 13, 19, 45, 00, TimeSpan.Zero),
                    Duration = TimeSpan.FromHours(5)
                }
            };

            // 例8-18 単純な投影
            var projected = from ev in events
                            select ev;

            PrintEvents(projected);


            AnonTypes(events);


            var eventsByday = from ev in events
                              group ev by ev.StartTime.Date;

            // 例8-21 複数のfrom句を使用して平坦なリストとして取得する
            var items = from day in eventsByday
                        from item in day
                        select item;

            var items2 = eventsByday.SelectMany(day => day);

            // 例8-22 複数のfrom句を使用する別の例
            int[] numbers = { 1, 2, 3, 4, 5 };
            var multiplied = from x in numbers
                             from y in numbers
                             select x * y;
            foreach (int n in multiplied)
            {
                Console.WriteLine(n);
            }

            // 例8-23 要素に番号をつける
            var numberedEvents = events.
                Select((ev, i) => string.Format("{0}: {1}", i + 1, ev.Title));
            foreach (string item in numberedEvents)
            {
                Console.WriteLine(item);
            }
        }

        private static void AnonTypes(List<CalendarEvent> events)
        {
            // 例8-19 匿名型を使用するselect句
            var projected = from ev in events
                            select new
                            {
                                Title = ev.Title,
                                StartTime = ev.StartTime,
                                EndTime = ev.StartTime + ev.Duration
                            };

            // 例8-20 匿名型を含むコレクションを使用する
            foreach (var item in projected)
            {
                Console.WriteLine("イベント {0} は {1} に開始して {2} に終了します。",
                item.Title, item.StartTime, item.EndTime);
            }
        }

        static dynamic Test()
        {
            return new { Prop1 = "value1", Prop2 = 2 };
        }

        static void PrintEvents(IEnumerable<CalendarEvent> events)
        {
            foreach (CalendarEvent item in events)
            {
                Console.WriteLine(item.Title + ": " + item.StartTime);
            }
        }
    }
}
