﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace QueriesAndExpressions
{
    class Program
    {
        // 例8-1 ファイル名の列挙
        static IEnumerable<string> GetAllFilesInDirectory(string directoryPath)
        {
            return Directory.EnumerateFiles(directoryPath, "*",
                SearchOption.AllDirectories);
        }

        static void ShowBigFilesUsingQueryExpression()
        {
            // 例8-2 列挙可能なオブジェクトに対してLINQを使用する
            var bigFiles = from file in GetAllFilesInDirectory(@"c:\")
                           where new FileInfo(file).Length > 10000000
                           select file;
            foreach (string file in bigFiles)
            {
                Console.WriteLine(file);
            }
        }

        static void ShowBigFilesUsingMethodCalls()
        {
            // 例8-3 メソッド呼び出しによるLINQクエリ
            var bigFiles = GetAllFilesInDirectory(@"c:\").
                Where(file => new FileInfo(file).Length > 10000000);

            foreach (string file in bigFiles)
            {
                Console.WriteLine(file);
            }
        }

        static void ShowBigFilesUsingMethodCallsWithSelect()
        {
            // 例8-4 WhereおよびSelectメソッド
            var bigFiles = GetAllFilesInDirectory(@"c:\").
                Where(file => new FileInfo(file).Length > 10000000).
                Select(file => "ファイル: " + file);

            foreach (string file in bigFiles)
            {
                Console.WriteLine(file);
            }
        }

        static void Main(string[] args)
        {
            ShowBigFilesUsingQueryExpression();
            ShowBigFilesUsingMethodCalls();
            ShowBigFilesUsingMethodCallsWithSelect();
        }
    }
}
