﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.ServiceModel.Channels;

namespace DictionaryExamples
{
    class Program
    {
        static void Main(string[] args)
        {
            var paramExpr = System.Linq.Expressions.Expression.Parameter(typeof(string), "hello");
            // 例9-4 ディクショナリ初期化リスト
            var myDictionary = new Dictionary<string, string>()
            {
                { "犬", "猫じゃない。" },
                { "海", "大きくて青い不安定なもので、中に人魚が住んでいる。" }
            };

            // 例9-1 ディクショナリの項目を検索
            string definition = myDictionary["海"];
            Console.WriteLine("海: " + definition);
        }

        static void InitWithNew()
        {
            // 例9-2 キーと値を文字列とするディクショナリ
            Dictionary<string, string> myDictionary = new Dictionary<string, string>();
        }

        static void InitWithVar()
        {
            // 例9-3 varキーワードにより冗長な繰り返しを避ける
            var myDictionary = new Dictionary<string, string>();
        }

        static void WcfDynamicProperties()
        {
            // 例9-6 WCFのMessageに対して動的プロパティを設定する
            Message wcfMessage = CreateMessageSomehow();
            HttpRequestMessageProperty reqProps = new HttpRequestMessageProperty();
            reqProps.Headers.Add(HttpRequestHeader.UserAgent, "my user agent");
            wcfMessage.Properties[HttpRequestMessageProperty.Name] = reqProps;
        }

        private static Message CreateMessageSomehow()
        {
            return Message.CreateMessage(MessageVersion.Soap11, "SomeAction");
        }


        class BuildingInfo { public int ID { get; set; } public string Name { get; set; } }
        class DS { public List<BuildingInfo> Buildings { get; set; } }

        static void LinqToDictionary()
        {
            DS MyDataSource = new DS
            {
                Buildings = 
                {
                    new BuildingInfo { ID = 1, Name = "The Arthur Dent Building" },
                    new BuildingInfo { ID = 2, Name = "Ford Prefect Hall" },
                    new BuildingInfo { ID = 2, Name = "Beeblebrox Towers" }
                }
            };

            // 例9-11 LINQのToDictionary演算子
            IDictionary<int, string> buildingIdToNameMap =
                MyDataSource.Buildings.ToDictionary(
                    building => building.ID,
                    building => building.Name);
        }
    }
}
