﻿using System.Collections.Generic;
using System.Linq;

namespace DictionaryExamples
{
    // 例9-5 ディクショナリをキャッシュとして使用する
    class RecordCache
    {
        private Dictionary<int, Record> cachedRecords =
        new Dictionary<int, Record>();
        public Record GetRecord(int recordId)
        {
            Record result;
            if (cachedRecords.TryGetValue(recordId, out result))
            {
                // キャッシュに見つかりましたが、これは失効しているでしょうか？
                if (IsStale(result))
                {
                    result = null;
                }
            }
            if (result == null)
            {
                result = LoadRecord(recordId);
                // 新しく読み込まれたレコードをキャッシュに追加します
                cachedRecords[recordId] = result;
            }
            DiscardAnyOldCacheEntries();
            return result;
        }
        private Record LoadRecord(int recordId)
        {
            // ... ここにレコードをロードするコードを記述します ...
            return new Record();
        }
        private bool IsStale(Record result)
        {
            // ... ここにコードが失効していないかを調べるコードを記述します ...
            return false;
        }

        // 例9-10 ディクショナリを入力とするLINQクエリ
        private void DiscardAnyOldCacheEntries()
        {
            // ソースに対してToList()を呼び出して、
            // コピーを対象とした反復処理を行います。
            // そうすることによって以降のforeachループ内では
            // Removeの呼び出しによる例外を回避できます。
            var staleKeys = from entry in cachedRecords.ToList()
                            where IsStale(entry.Value)
                            select entry.Key;
            foreach (int staleKey in staleKeys)
            {
                cachedRecords.Remove(staleKey);
            }
        }
    }
}
