﻿using System;
using System.Linq;
using System.Text;

namespace ManipulatingText
{
    class Program
    {
        // 例10-62 乱雑な入力のシミュレーション
        private static string[] SoliloquizeLikeAUser()
        {
            return new string[] {
                "",
                null,
                " ",
                String.Empty,
                " To be, or not to be--that is the question: ",
                "Whether 'tis nobelr in the mind to suffer,",
                "\tThe slings and arrows of outrageous fortune ,",
                "",
                "\tOr to take arms against a sea of troubles, ",
                "And by opposing end them.",
                "",
                "",
                "",
                "",
                ""};
        }

        static void Main(string[] args)
        {
            {
                // 例10-63 入力の整理
                string[] strings = SoliloquizeLikeAUser();
                string output = String.Empty; // これは""と同等です
                foreach (string line in strings)
                {
                    // この位置で各行に対して処理を行います...
                    // そして...
                    output = output + line + Environment.NewLine;
                }
                Console.WriteLine(output);
            }

            Console.WriteLine("-------");

            {
                // 例10-64 StringBuilder による文字列の作成
                // および
                // 例10-73 テキストを整理するための以前のコード
                string[] strings = SoliloquizeLikeAUser();
                StringBuilder output = new StringBuilder();
                foreach (string line in strings)
                {
                    // 例10-75 空行の検出
                    //if (line != String.Empty)
                    // 例10-76 空かnull に一致するかどうか確認する
                    if (!String.IsNullOrEmpty(line))
                    {
                        output.AppendLine(line);
                    }
                    else
                    {
                        System.Diagnostics.Debug.WriteLine("空の行を検出しました。");
                    }
                }

                // 例10-74 特定のタイプミスを修正
                output.Replace("nobelr", "nobler");
                Console.WriteLine(output.ToString());
            }

            Console.WriteLine("-------");

            {
                {
                    string[] strings = SoliloquizeLikeAUser();
                    StringBuilder output = new StringBuilder();
                    // 例10-77 空白の削除
                    foreach (string line in strings)
                    {
                        if (line != null)
                        {
                            //string trimmedLine = line.Trim();
                            // 例10-78 指定した文字の削除
                            string trimmedLine = line.Trim(' ', '\t', ',');
                            if (trimmedLine.Length != 0)
                            {
                                output.AppendLine(trimmedLine);
                            }
                            else
                            {
                                System.Diagnostics.Debug.WriteLine(
                                "空の行を検出しました。(空白削除後）");
                            }
                        }
                        else
                        {
                            System.Diagnostics.Debug.WriteLine("null行を検出しました。");
                        }
                    }

                    output.Replace("nobelr", "nobler");
                    Console.WriteLine(output.ToString());
                }
            }
        }

        // 例10-79 すべての空白と追加指定された文字の削除
        private static string TrimWhitespaceAnd(
            string inputString,
            params char[] characters)
        {
            int start = 0;
            while (start < inputString.Length)
            {
                // 空白でもなく、削除リストの文字でもないなら
                // 削除しない最初の文字が見つかったことになるのでループを停止します
                if (!char.IsWhiteSpace(inputString[start]) &&
                !characters.Contains(inputString[start]))
                {
                    break;
                }
                // 文字を1つ先に進めます
                start++;
            }
            // 末尾から先頭に向けて調査します
            int end = inputString.Length - 1;
            while (end >= start)
            {
                // 空白でもなく、削除リストの文字でもないなら
                // 削除しない最初の文字が見つかったことになります
                if (!char.IsWhiteSpace(inputString[end]) &&
                !characters.Contains(inputString[end]))
                {
                    break;
                }
                // 文字を1つ前に進めます
                end--;
            }
            // Substringに指定できるよう
            // 文字列の長さを調べます
            int length = (end - start) + 1;
            if (length == inputString.Length)
            {
                // 削除された文字がなければ
                // 入力された文字列をそのまま返します
                // (新しい文字列を生成しません)
                return inputString;
            }
            // 長さ0ならば空文字列を返します
            if (length == 0)
            {
                return string.Empty;
            }
            return inputString.Substring(start, length);
        }
    }
}
