﻿using System;
using System.IO;
using System.Security.Cryptography;

namespace Crypto
{
    class Program
    {
        // 例11-51 暗号ストリームの利用
        static void Main(string[] args)
        {
            byte[] key;
            byte[] iv;
            // アルゴリズム用に適切なキーと初期化ベクトルを取得します
            SelectKeyAndIV(out key, out iv);
            string superSecret = "これは重大な秘密です";
            Console.WriteLine(superSecret);
            string encryptedText = EncryptString(superSecret, key, iv);
            Console.WriteLine(encryptedText);
            string decryptedText = DecryptString(encryptedText, key, iv);
            Console.WriteLine(decryptedText);
            Console.ReadKey();
        }

        // 例11-52 キーとIV の作成
        private static void SelectKeyAndIV(out byte[] key, out byte[] iv)
        {
            var algorithm = TripleDES.Create();
            algorithm.GenerateIV();
            algorithm.GenerateKey();
            key = algorithm.Key;
            iv = algorithm.IV;
        }

        // 例11-53 文字列の暗号化
        private static string EncryptString(string plainText, byte[] key, byte[] iv)
        {
            // TripleDESアルゴリズム用の暗号化サービスプロバイダを作成します。
            var serviceProvider = new TripleDESCryptoServiceProvider();

            using (MemoryStream memoryStream = new MemoryStream())
            using (var cryptoStream = new CryptoStream(
                                            memoryStream,
                                            serviceProvider.CreateEncryptor(key, iv),
                                            CryptoStreamMode.Write))
            using (StreamWriter writer = new StreamWriter(cryptoStream))
            {
                // 暗号ストリームにテキストを書き込みつつ、暗号化を行います。
                writer.Write(plainText);
                // writerが暗号化ストリームの内容を
                // バッファからディスクへ確実に書き出すようにします。
                writer.Flush();
                // 暗号化ストリームが最終ブロックもバッキングストアとなっている
                // ストリームに書き出すようにする必要があります。
                // そうでないと一部の内容を失うことになるかもしれません...
                cryptoStream.FlushFinalBlock();

                // メモリストリームに書き込まれた暗号化ストリームを返します
                return GetCipherText(memoryStream);
            }
        }

        // 例11-54 Base64 への変換
        private static string GetCipherText(MemoryStream memoryStream)
        {
            byte[] buffer = memoryStream.ToArray();
            return System.Convert.ToBase64String(buffer, 0, buffer.Length);
        }

        // 例11-55 復号化
        private static string DecryptString(string cipherText, byte[] key, byte[] iv)
        {
            // TripleDESアルゴリズム用の暗号サービスプロバイダを作成します。
            var serviceProvider = new TripleDESCryptoServiceProvider();

            // Base64エンコード文字列をデコードし、暗号文のバイト列を取得します。
            byte[] cipherTextBytes = Convert.FromBase64String(cipherText);

            // これらのバイト列に対しメモリストリームを作成します。
            using (MemoryStream memoryStream = new MemoryStream(cipherTextBytes))
            // メモリストリームに対し、指定したアルゴリズムを使用して
            // 暗号ストリームを作成します
            // (用意されたキーと初期化ベクトル(IV)を使用します)。
            using (var cryptoStream =
                new CryptoStream(
                    memoryStream,
                    serviceProvider.CreateDecryptor(key, iv),
                    CryptoStreamMode.Read))
            // 最後に、ストリームに対してStreamReaderを作成し、
            // 原文を復元します。
            using (StreamReader reader = new StreamReader(cryptoStream))
            {
                return reader.ReadToEnd();
            }
        }
    }
}
