﻿using System;
using System.IO;
using System.IO.IsolatedStorage;

namespace Isolated
{
    class Program
    {
        // 例11-49 ストア内でフォルダとファイルを作成
        static void Main(string[] args)
        {
            IsolatedStorageFile store = IsolatedStorageFile.GetUserStoreForAssembly();
            // ディレクトリを作成します。複数回呼んでも問題ありません。
            store.CreateDirectory("Settings");
            // ファイルが既に存在すれば開き、ない場合には新規作成します。
            using (IsolatedStorageFileStream stream = store.OpenFile(
                                    "Settings\\standardsettings.txt",
                                    System.IO.FileMode.OpenOrCreate,
                                    System.IO.FileAccess.ReadWrite))
            {
                UseStream(stream);
            }

            Console.ReadKey();
        }

        // 例11-50 StreamReaderとStreamWriterを分離ストレージに対して使用する
        static void UseStream(Stream stream)
        {
            if (stream.Length > 0)
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            else
            {
                using (StreamWriter writer = new StreamWriter(stream))
                {
                    writer.WriteLine(
                        "{0}に設定を初期化しました。", DateTime.Now.TimeOfDay);
                    Console.WriteLine("設定が初期化されました。");
                }
            }
        }
    }
}
