﻿using System;
using System.IO;

namespace AsyncIo
{
    class Program
    {
        // 例11-47 非同期ファイルI/O
        static void Main(string[] args)
        {
            string path = "mytestfile.txt";
            // テストファイルの作成
            //using (var file = File.Create(path, 4096, FileOptions.Asynchronous))
            // 例11-48 非同期的な振る舞いを要求しない
            using (var file = File.Create(path, 4096, FileOptions.None))
            {
                // 書き込むバイト列
                byte[] myBytes = new byte[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
                IAsyncResult asyncResult = file.BeginWrite(
                    myBytes,
                    0,
                    myBytes.Length,
                    // 匿名デリゲートとして書かれたコールバック関数
                    delegate(IAsyncResult result)
                    {
                        // 「必ず」EndWrite()を1回だけ呼ばなければいけません
                        file.EndWrite(result);
                        // そして任意の処理を行います
                        Console.WriteLine(
                            "処理の完了時にスレッド{0}で呼び出されました。",
                            System.Threading.Thread.CurrentThread.ManagedThreadId);
                    },
                    null);

                // 待っている間に何か別のことをできます...
                Console.WriteLine(
                    "スレッド{0}で完了を待機しています...",
                    System.Threading.Thread.CurrentThread.ManagedThreadId);
                // メインスレッドで待機します
                asyncResult.AsyncWaitHandle.WaitOne();
                Console.WriteLine(
                    "スレッド{1}で{0}に完了...",
                    asyncResult.CompletedSynchronously ?
                        "同期的" : "非同期的",
                    System.Threading.Thread.CurrentThread.ManagedThreadId);
                Console.ReadKey();
                return;
            }
        }
    }
}
