﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace DuplicateFinder
{
    class Program
    {
        // 例11-1 FindDuplicateFilesプログラムのMainメソッド
        static void Main(string[] args)
        {
            bool recurseIntoSubdirectories = false;

            if (args.Length < 1)
            {
                ShowUsage();
                return;
            }

            int firstDirectoryIndex = 0;

            if (args.Length > 1)
            {
                // 再帰処理が必要かどうかチェック
                if (args[0] == "/sub")
                {
                    if (args.Length < 2)
                    {
                        ShowUsage();
                        return;
                    }
                    recurseIntoSubdirectories = true;
                    firstDirectoryIndex = 1;
                }
            }

            // コマンドラインからディレクトリのリストを取得
            var directoriesToSearch = args.Skip(firstDirectoryIndex);

            List<FileNameGroup> filesGroupedByName =
                InspectDirectories(recurseIntoSubdirectories, directoriesToSearch);

            DisplayMatches(filesGroupedByName);

            Console.ReadKey();
        }


        // 例11-2 コマンドラインの使用方法を出力
        private static void ShowUsage()
        {
            Console.WriteLine("重複ファイル検索");
            Console.WriteLine("================");
            Console.WriteLine(
                "1つ以上のディレクトリに重複ファイルがあるかどうかをチェックします。");
            Console.WriteLine();
            Console.WriteLine(
                "使用方法: findduplicatefiles [/sub] ディレクトリ名 [ディレクトリ名] ...");
            Console.WriteLine("/sub - サブディレクトリ以下を再帰的に探索します");
            Console.ReadKey();
        }

        // 例11-4 InspectDirectoriesメソッド
        private static List<FileNameGroup> InspectDirectories(
            bool recurseIntoSubdirectories,
            IEnumerable<string> directoriesToSearch)
        {
            // 例11-5 ディレクトリ内のファイル名を取得
            var searchOption = recurseIntoSubdirectories ?
                SearchOption.AllDirectories : SearchOption.TopDirectoryOnly;

            // 検索対象の全てのディレクトリに含まれるすべてのファイルパスを取得します
            var allFilePaths = from directory in directoriesToSearch
                               from file in Directory.GetFiles(directory, "*.*",
                                                                searchOption)
                               select file;

            // ローカルファイル名(親ディレクトリのパスを含まないファイル名)で
            // ファイルをグループ化しつつ、同名のローカルファイル名を持つ
            // 各ファイルの詳細を含んだリストを作成します。

            // 例11-7 フルパスではないファイル名の取得
            var fileNameGroups = from filePath in allFilePaths
                                 let fileNameWithoutPath = Path.GetFileName(filePath)
                                 group filePath by fileNameWithoutPath into nameGroup
                                 // 例11-8 ファイルサイズの取得
                                 select new FileNameGroup
                                 {
                                     FileNameWithoutPath = nameGroup.Key,
                                     FilesWithThisName =
                                      (from filePath in nameGroup
                                       let info = new FileInfo(filePath)
                                       select new FileDetails
                                       {
                                           FilePath = filePath,
                                           FileSize = info.Length
                                       }).ToList()
                                 };

            return fileNameGroups.ToList();
        }

        // 例11-9 DisplayMatches
        private static void DisplayMatches(
            IEnumerable<FileNameGroup> filesGroupedByName)
        {
            var groupsWithMoreThanOneFile = from nameGroup in filesGroupedByName
                                            where nameGroup.FilesWithThisName.Count > 1
                                            select nameGroup;

            foreach (var fileNameGroup in groupsWithMoreThanOneFile)
            {
                // 見つかったファイルをサイズでグループ化して、
                // 同じサイズのファイルが1つ以上ある場合には選択します
                var matchesBySize = from file in fileNameGroup.FilesWithThisName
                                    group file by file.FileSize into sizeGroup
                                    where sizeGroup.Count() > 1
                                    select sizeGroup;

                foreach (var matchedBySize in matchesBySize)
                {
                    string fileNameAndSize = string.Format("{0} ({1} bytes)",
                    fileNameGroup.FileNameWithoutPath, matchedBySize.Key);
                    WriteWithUnderlines(fileNameAndSize);
                    // 重複ファイルを含むそれぞれのディレクトリを表示
                    foreach (var file in matchedBySize)
                    {
                        Console.WriteLine(Path.GetDirectoryName(file.FilePath));
                    }
                    Console.WriteLine();
                }
            }
        }

        private static void WriteWithUnderlines(string text)
        {
            Console.WriteLine(text);
            Console.WriteLine(new string('-', text.Length));
        }
    }
}
