﻿using System;
using System.IO;
using System.Net;
using System.Net.Cache;
namespace HttpRequestResponse
{
    class Program
    {
        static void Main(string[] args)
        {
            {
                // 例13-13 HttpWebRequestとHttpWebResponseを使用してデータを文字列として取得する
                HttpWebRequest req = (HttpWebRequest) WebRequest.Create("http://oreilly.com/");
                using (HttpWebResponse resp = (HttpWebResponse) req.GetResponse())
                using (Stream respStream = resp.GetResponseStream())
                using (StreamReader reader = new StreamReader(respStream))
                {
                    string pageContent = reader.ReadToEnd();
                    Console.WriteLine(pageContent);
                }
            }

            {
                // 例13-14 HttpWebRequestを使用してユーザーエージェントヘッダを変更する
                HttpWebRequest req = (HttpWebRequest) WebRequest.Create("http://oreilly.com/");
                req.UserAgent = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us)"
                    + " AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11a";
                using (HttpWebResponse resp = (HttpWebResponse) req.GetResponse())
                using (Stream respStream = resp.GetResponseStream())
                using (StreamReader reader = new StreamReader(respStream))
                {
                    string pageContent = reader.ReadToEnd();
                    Console.WriteLine(pageContent);
                }
            }

            {
                // 例13-15 レスポンスを非同期的に取得する
                HttpWebRequest req = (HttpWebRequest) WebRequest.Create("http://oreilly.com/");
                req.BeginGetResponse(delegate(IAsyncResult asyncResult)
                {
                    using (HttpWebResponse resp = (HttpWebResponse)
                    req.EndGetResponse(asyncResult))
                    using (Stream respStream = resp.GetResponseStream())
                    using (StreamReader reader = new StreamReader(respStream))
                    {
                        string pageContent = reader.ReadToEnd();
                        Console.WriteLine(pageContent);
                    }
                }, null);

                Console.ReadKey();
            }
        }

        static void Credentials()
        {
            {
                // 例13-16 統合認証を行えるようにする
                HttpWebRequest request =
                    (HttpWebRequest) WebRequest.Create("http://intraweb/");
                request.Credentials = CredentialCache.DefaultCredentials;
            }

            {
                // 例13-17 ベーシック認証またはダイジェスト認証用にCredentialsを設定する
                HttpWebRequest request =
                    (HttpWebRequest) WebRequest.Create("https://intraweb/");
                request.Credentials = new NetworkCredential("user1", "p@ssw0rd");
            }
        }

        static void Proxy()
        {
            // 例13-19 明示的にプロキシを設定する
            HttpWebRequest request =
            (HttpWebRequest) WebRequest.Create("https://intraweb/");
            request.Proxy = new WebProxy("http://corpwebproxy/");
        }

        static void Cache()
        {
            // 例13-20 キャッシュポリシーの設定
            HttpRequestCachePolicy cachePolicy = new HttpRequestCachePolicy(
            HttpRequestCacheLevel.CacheIfAvailable);
            HttpWebRequest request =
            (HttpWebRequest) WebRequest.Create("https://intraweb/");
            request.CachePolicy = cachePolicy;
        }

        static void Cookies()
        {
            // 例13-21 レスポンスからクッキーを取得する
            CookieContainer container = new CookieContainer();

            Uri address = new Uri("http://amazon.com/");
            HttpWebRequest req = (HttpWebRequest) WebRequest.Create(address);
            HttpWebResponse resp = (HttpWebResponse) req.GetResponse();

            CookieCollection cookies = resp.Cookies;
            container.Add(address, cookies);

            SendCookies(container);
        }

        static void SendCookies(CookieContainer container)
        {
            Uri address = new Uri("http://oreilly.com/");
            HttpWebRequest newReq = (HttpWebRequest) WebRequest.Create(address);
            newReq.CookieContainer = container;
        }
    }
}
