﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;

namespace CpuUsageComparison
{
    // 例16-8 マルチスレッド 対 シングルスレッド
    class Program
    {
        static int Count;
        const int Iterations = 100000000;
        static readonly int ProcessorCount = Environment.ProcessorCount;
        
        static void Go()
        {
            for (int i = 0; i < Iterations; ++i)
            {
                Interlocked.Increment(ref Count);
            }
        }

        static void GoSingle(int repeat)
        {
            for (int outer = 0; outer < repeat; ++outer)
            {
                for (int i = 0; i < Iterations; ++i)
                {
                    Count += 1;
                }
            }
        }

        static void Main(string[] args)
        {
            RunMultithreaded();
            RunSinglethreaded();
        }

        private static void RunMultithreaded()
        {
            Stopwatch sw = new Stopwatch();
            sw.Start();

            Count = 0;
            List<Thread> threads = (from i in Enumerable.Range(0, ProcessorCount)
                                    select new Thread(Go)).ToList();
            threads.ForEach(t => t.Start());
            threads.ForEach(t => t.Join());

            sw.Stop();
            Console.WriteLine("マルチスレッド処理完了。カウント: {0}, 時間: {1}", Count, sw.Elapsed);
        }

        private static void RunSinglethreaded()
        {
            Stopwatch sw = new Stopwatch();
            sw.Start();

            Count = 0;
            GoSingle(ProcessorCount);

            sw.Stop();
            Console.WriteLine("シングルスレッド処理完了。カウント: {0}, 時間: {1}", Count, sw.Elapsed);
        }
    }
}
