﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace MonitorExamples
{
    class Program
    {
        // 例16-10 MostRecentlyUsed クラスのテスト
        const int Iterations = 10000;

        static void TestMru(MostRecentlyUsed mru)
        {
            // 各スレッドが確実に違うデータを保持するように、
            // 乱数生成器をスレッドIDで初期化します。
            // (ただしこれによりテストを実行する度に違う値になるので、
            // 理想的ではないかもしれません。
            Random r = new Random(Thread.CurrentThread.ManagedThreadId);
            string[] items = { "One", "Two", "Three", "Four", "Five",
                                 "Six", "Seven", "Eight" };
            for (int i = 0; i < Iterations; ++i)
            {
                mru.UseItem(items[r.Next(items.Length)]);
            }
        }

        static void Main(string[] args)
        {
            // 例16-11 マルチスレッド版のテスト
            MostRecentlyUsed mru = new MostRecentlyUsed(4);

            const int TestThreadCount = 2;
            List<Thread> threads = (from i in Enumerable.Range(0, TestThreadCount)
                                    select new Thread(() => TestMru(mru))).ToList();

            threads.ForEach(t => t.Start());
            threads.ForEach(t => t.Join());

            foreach (string item in mru.GetItems())
            {
                Console.WriteLine(item);
            }
        }
    }
}
