﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace NonInteropDynamic
{
    class Static
    {
        // 例18-21 簡単なフィルタ
        static bool Test(int x)
        {
            return x < 100;
        }

        public static void Test()
        {
            // 例18-22 LINQ を使ったフィルタ
            var nums = Enumerable.Range(1, 200);
            var filteredNumbers = nums.Where(Test);
        }
    }

    class Dynamic
    {
        // 例18-23 動的フィルタ
        static bool Test(dynamic x)
        {
            return x < 100;
        }

        // 例18-25 dynamic と相性のいいWhere 実装
        static IEnumerable<T> DynamicWhere<T>(IEnumerable<T> input, dynamic test)
        {
            foreach (T item in input)
            {
                if (test(item))
                {
                    yield return item;
                }
            }
        }

        public static void Test()
        {
            var nums = Enumerable.Range(1, 200);

            {
                // 例18-24 LINQ に適合する動的フィルタ
                var filteredNumbers = nums.Where(x => Test(x));
            }


            // 例18-26のコンパイルが失敗する様子を確認するためには以下のfalseをtrueに変更します。
#if false
            {
                // 例18-26 DynamicWhere の呼び出しを試みる（が失敗する）
                var filteredNumbers = DynamicWhere(nums, Test); // コンパイルエラー
            }
#else
            {
                // 例18-27 DynamicWhere に手がかりを与える
                var filteredNumbers = DynamicWhere(nums, (Predicate<dynamic>) Test);
            }
#endif
        }
    }

    class Program
    {

        static void Main(string[] args)
        {
            Static.Test();
            Dynamic.Test();
        }
    }
}
