﻿using System;
using System.Linq;
using AvWorksBrowser.AdvWorksData;
using System.Collections.Generic;

namespace AvWorksBrowser
{
    public class CategoryViewModel
    {
        private ProductCategory entity;

        private CategoryViewModel(ProductCategory categoryEntity)
        {
            entity = categoryEntity;
        }

        public static void GetCategories(Action<IList<CategoryViewModel>> categoriesFetchedCallback)
        {
            Uri dataUri = new Uri(App.Current.Host.Source, "/AdventureWorksData.svc");
            AdventureWorksLT2008Entities dataServiceContext = new AdventureWorksLT2008Entities(dataUri);

            // Expand("Product")は「もしあれば、関連するProductエンティティのデータをすべてください」という指示を表します。
            // これはEntity FrameworkのIncludeメソッドに似ています(なぜ違う名前がつけられたのかわかりませんが)。

            // 実際には検索結果をフィルタできるようになっているべきですが、
            // そのためには以下のようなLINQクエリを記述しなければいけなくなります：
            // ProductCategory()?$filter=Product/Count gt 0&$expand=Products
            // しかしADO.NETのDSはProduct/Countという部分を理解できないため
            // 404が返されます。従って、返された結果についてはクライアント上で
            // フィルタする必要があります。
            var q = dataServiceContext.ProductCategories.Expand("Products");
            q.BeginExecute(delegate(IAsyncResult iar)
            {
                var categories = q.EndExecute(iar);
                var categoryViewModels = from category in categories
                                         where category.Products.Count > 0
                                         select new CategoryViewModel(category);

                categoriesFetchedCallback(categoryViewModels.ToArray());
            }, null);

        }

        public string DisplayName { get { return entity.Name; } }

        private ProductViewModel[] cachedProducts;

        public IList<ProductViewModel> Products
        {
            get
            {
                if (cachedProducts == null)
                {
                    cachedProducts = (from product in entity.Products
                                      select new ProductViewModel(product)).ToArray();
                }

                return cachedProducts;
            }
        }

    }
}
